Package: dada2
Type: Package
Title: Accurate, high-resolution sample inference from amplicon
        sequencing data
Description: The dada2 package infers exact amplicon sequence variants (ASVs) from 
    high-throughput amplicon sequencing data, replacing the coarser and less accurate
    OTU clustering approach. The dada2 pipeline takes as input demultiplexed fastq 
    files, and outputs the sequence variants and their sample-wise abundances after 
    removing substitution and chimera errors. Taxonomic classification is available
    via a native implementation of the RDP naive Bayesian classifier, and species-level
    assignment to 16S rRNA gene fragments by exact matching.
Version: 1.40.0
Date: 2024-12-01
Maintainer: Benjamin Callahan <benjamin.j.callahan@gmail.com>
Author: Benjamin Callahan <benjamin.j.callahan@gmail.com>, Paul McMurdie, Susan Holmes
License: LGPL-2
LazyLoad: yes
Depends: R (>= 4.1.0), Rcpp (>= 0.12.0), methods (>= 3.4.0)
Imports: Biostrings (>= 2.42.1), ggplot2 (>= 2.1.0), reshape2 (>=
        1.4.1), ShortRead (>= 1.32.0), RcppParallel (>= 4.3.0),
        parallel (>= 3.2.0), IRanges (>= 2.6.0), XVector (>= 0.16.0),
        BiocGenerics (>= 0.22.0)
Suggests: BiocStyle, knitr, rmarkdown
LinkingTo: Rcpp, RcppParallel
SystemRequirements: GNU make
VignetteBuilder: knitr
biocViews: ImmunoOncology, Microbiome, Sequencing, Classification,
        Metagenomics
URL: http://benjjneb.github.io/dada2/
BugReports: https://github.com/benjjneb/dada2/issues
LazyData: true
Collate: 'RcppExports.R' 'allClasses.R' 'allPackage.R' 'chimeras.R'
        'dada.R' 'errorModels.R' 'filter.R' 'misc.R' 'multiSample.R'
        'paired.R' 'plot-methods.R' 'sequenceIO.R' 'show-methods.R'
        'taxonomy.R'
RoxygenNote: 7.3.1
git_url: https://git.bioconductor.org/packages/dada2
git_branch: RELEASE_3_23
git_last_commit: 52cd78e
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 21:35:10 UTC; biocbuild
